import tempfile, os

from .download import get_videos_list, download_vtts, vtt_to_db
from .db_utils import add_channel_info, get_num_vids, get_vid_ids_by_channel_id

def update_channel(channel_id, channel_name, language, number_of_jobs, s):
    """
    Downloads all the videos from a channel to a tmp directory
    """
    with tempfile.TemporaryDirectory() as tmp_dir:

        channel_url = f"https://www.youtube.com/channel/{channel_id}/videos"

        public_video_ids = get_videos_list(channel_url)
        num_public_vids = len(public_video_ids)
        num_local_vids = get_num_vids(channel_id)

        if num_public_vids == num_local_vids:
            print("No new videos to download")
            exit()

        local_vid_ids = get_vid_ids_by_channel_id(channel_id)
        local_vid_ids = [i[0] for i in local_vid_ids]


        fresh_videos = [i for i in public_video_ids if i not in local_vid_ids]

        print(f"Found {len(fresh_videos)} videos on \"{channel_name}\" not in the database")
        print(f"Downloading {len(fresh_videos)} new videos from \"{channel_name}\"")

        download_vtts(number_of_jobs, fresh_videos, language, tmp_dir)

        vtt_to_parse = os.listdir(tmp_dir)
        if len(vtt_to_parse) == 0:
            print("No new videos saved")
            print(f"{len(fresh_videos)} videos on \"{channel_name}\" do not have subtitles")
            exit()

        vtt_to_db(tmp_dir)

        print(f"Added {len(vtt_to_parse)} new videos from \"{channel_name}\" to the database")


from pprint import pprint
from .download import get_channel_id_from_input
from .db_utils import * 
from .utils import *
from rich.console import Console
from rich.text import Text


# full text search
def fts_search(text, scope, channel_id=None, video_id=None, limit=None):
    """
    Calls search functions and prints the results 
    """
    console = Console()

    if scope == "all":
        res = search_all(text, limit)
    
    if scope == "channel":
        channel_id = get_channel_id_from_input(channel_id)
        res = search_channel(channel_id, text, limit)
    
    if scope == "video":
        res = search_video(video_id, text, limit)

    if len(res) == 0:
        console.print("- Try shortening the search to specific words")
        console.print("- Try using the wildcard operator [bold]*[/bold] to search for partial words")
        console.print("- Try using the [bold]OR[/bold] operator to search for multiple words")
        if len(text.split(" ")) > 1:
            example_or = text.replace(" ", " OR ")
            console.print(f"    - EX: \"[bold]{example_or}[/bold]\"")
        else: 
            console.print(f"    - EX: \"[bold]foo OR [bold]bar[/bold]\"")
        exit()

    return res


# pretty print search results
def print_fts_res(res, query):

    console = Console()

    fts_res = []
    channel_names = []

    for quote in res:
        quote_match = {}
        video_id = quote["video_id"]
        time_stamp = quote["start_time"]
        time = time_to_secs(time_stamp)
        link = f"https://youtu.be/{video_id}?t={time}"

        quote_match["channel_name"] = get_channel_name_from_video_id(video_id)
        channel_names.append(quote_match["channel_name"])

        quote_match["video_title"] = get_title_from_db(video_id)
        quote_match["subs"] = bold_query_matches(quote["text"].strip(), query)
        quote_match["time_stamp"] = time_stamp
        quote_match["video_id"] = video_id
        quote_match["link"] = link 

        fts_res.append(quote_match)


    """
    need to resturcutre the data to be able to print it in a nice way

    fts_dict = {
        "channel_name": {
            "video_name": [
                {
                    "quote": "quote",
                    "time_stamp": "time_stamp",
                    "link": "link"
                }
            ]
        }
    }

    original format is:
    fts_res = [
        {
            "channel_name": "channel_name",
            "video_name": "video_name",
            "quote": "quote",
            "time_stamp": "time_stamp",
            "link": "link"
        }
    ]
    """

    fts_dict = {}
    for quote in fts_res:
        channel_name = quote["channel_name"]
        video_name = quote["video_title"]
        quote_data = {
            "quote": quote["subs"],
            "time_stamp": quote["time_stamp"],
            "link": quote["link"]
        }
        if channel_name not in fts_dict:
            fts_dict[channel_name] = {}
        if video_name not in fts_dict[channel_name]:
            fts_dict[channel_name][video_name] = []
        fts_dict[channel_name][video_name].append(quote_data)
    

    # Sort the list by the total number of quotes in each channel
    channel_list = list(fts_dict.items())
    channel_list.sort(key=lambda x: sum(len(quotes) for quotes in x[1].values()))

    for channel_name, videos in channel_list:
        console.print(f"[spring_green2][bold]{channel_name}[/bold][/spring_green2]")
        console.print("")

        # Sort the list by the number of quotes in each video
        video_list = list(videos.items())
        video_list.sort(key=lambda x: len(x[1]))

        for video_name, quotes in video_list:
            console.print(f"    [bold][blue]{video_name}[/blue][/bold]")
            console.print("")

            # Sort the quotes by timestamp
            quotes.sort(key=lambda x: x['time_stamp'])

            for quote in quotes:
                link = quote["link"]
                time_stamp = quote["time_stamp"]
                words = quote["quote"]
                console.print(f"       [grey62][link={link}]{time_stamp}[/link][/grey62] -> [italic][white]\"{words}\"[/white][/italic]")
            console.print("")

    num_matches = len(res)
    num_channels = len(set(channel_names))  
    num_videos = len(set([quote["video_id"] for quote in res]))

    summary_str = f"Found [bold]{num_matches}[/bold] matches in [bold]{num_videos}[/bold] videos from [bold]{num_channels}[/bold] channel"

    if num_channels > 1:
        summary_str += "s"

    console.print(summary_str) 


import unittest
from pprint import pprint
from unittest.mock import patch, call
import subprocess
from yt_fts.download import get_playlist_urls


class TestGetPlaylistUrls(unittest.TestCase):
    def test_bob_ross(self):
        # bobross 
        playlist_url = "https://www.youtube.com/playlist?list=PLAEQD0ULngi69x_7JbQvSMprLRK_KSVLu"

        # expected_output = ['oh5p5f5_-7A', 'RInDWhYceLU', 'UOziR7PoVco', '0pwoixRikn4', 
        #                    'DFSIQNjKRfk', 'loAzRUzx1wI', 'sDdpc8uisD0', 'kQlFwTOkYzg', 
        #                    'QxcS7p1VHyQ', 'wDnLlywAL5I', 'Q03YvknOVe0', '4E35-8x_y04', 
        #                    'IEQWfszfRlA']

        actual_output = get_playlist_urls(playlist_url)
        print(actual_output)
        # self.assertEqual(actual_output, expected_output)
        self.assertEqual(len(actual_output), 13)


if __name__ == '__main__':
    unittest.main()



import unittest
from openai import OpenAI
import os
from yt_fts.vector_search import search_chroma_db

class TestSearchChromaDb(unittest.TestCase):
    def test_search_chroma_db(self):
        # Setup
        text = "nural network"
        scope = "all"
        video_id = None
        channel_id = None
        limit = 5
        client = OpenAI(api_key=os.environ["OPENAI_API_KEY"])

        result = search_chroma_db(text, scope, channel_id, video_id, limit, client)
        self.assertEqual(len(result), 5)
    
    def test_search_chroma_db_channel(self):
        text = "nural network"
        scope = "channel"
        video_id = None
        channel_id = "1"
        limit = 5
        client = OpenAI(api_key=os.environ["OPENAI_API_KEY"])

        result = search_chroma_db(text, scope, channel_id, video_id, limit, client)
        self.assertEqual(len(result), 5)

if __name__ == '__main__':
    unittest.main()


import sqlite3

from sqlite_utils import Database
from rich.console import Console
from rich.table import Table

from .utils import show_message
from .config import get_db_path 


def make_db(db_path):
    db = Database(db_path)

    db["Channels"].create({
            "channel_id": str,
            "channel_name": str,
            "channel_url": str,
        }, 
        pk="channel_id", 
        not_null={"channel_name", "channel_url"}, 
        if_not_exists=True
    )

    db["Videos"].create({
            "video_id": str,
            "video_title": str,
            "video_url": str,
            "channel_id": str
        }, 
        pk="video_id", 
        not_null={"video_title", "video_url"}, 
        if_not_exists=True, 
        foreign_keys=[
            ("channel_id", "Channels")
        ]
    )

    db["Subtitles"].create(
        {
            "subtitle_id": int,
            "video_id": str,
            "start_time": str,
            "stop_time": str,
            "text": str
        }, 
        pk="subtitle_id", 
        not_null={"start_time", "text"}, 
        if_not_exists=True, 
        foreign_keys=[
            ("video_id", "Videos")
        ]
    ).enable_fts(
        ["text"], 
        create_triggers=True, 
        replace=True
    )

    db["SemanticSearchEnabled"].create(
        {
            "channel_id": str,
        },
        if_not_exists=True,
        foreign_keys=[
            ("channel_id", "Channels")
        ]
        
    )


def add_channel_info(channel_id, channel_name, channel_url):
    
    db = Database(get_db_path())

    db["Channels"].insert({
        "channel_id": channel_id,
        "channel_name": channel_name,
        "channel_url": channel_url
    })


def add_video(channel_id, video_id,  video_title, video_url):
    
        conn = sqlite3.connect(get_db_path())
        cur = conn.cursor()
        existing_video = cur.execute("SELECT * FROM Videos WHERE video_id = ?", 
                                     (video_id,)).fetchone()

        if existing_video is None:
            cur.execute("INSERT INTO Videos (video_id, video_title, video_url, channel_id) VALUES (?, ?, ?, ?)",
                        (video_id, video_title, video_url, channel_id))
            conn.commit()

        else:
            print(f"{video_id} Video already exists in the database.")
        conn.close()


def add_subtitle(video_id, start_time, text):
    
    db = Database(get_db_path())

    db["Subtitles"].insert({
        "video_id": video_id,
        "timestamp": start_time,
        "text": text
    })


def get_channels():
    
    db = Database(get_db_path())

    return db.execute("SELECT ROWID, channel_id, channel_name, channel_url FROM Channels").fetchall()


def search_channel(channel_id, text, limit=None):
    
    db = Database(get_db_path())

    return list(db["Subtitles"].search(text, 
                                       where=f"video_id IN (SELECT video_id FROM Videos WHERE channel_id = '{channel_id}')",
                                       limit=limit))


def search_video(video_id, text, limit=None):
    
    db = Database(get_db_path())

    return list(db["Subtitles"].search(text, 
                                       where=f"video_id = '{video_id}'",
                                       limit=limit))

def search_all(text, limit=None):
    
    db = Database(get_db_path())

    return list(db["Subtitles"].search(text, limit=limit))


def get_title_from_db(video_id):

    db = Database(get_db_path())

    return db.execute(f"SELECT video_title FROM Videos WHERE video_id = ?", [video_id]).fetchone()[0]


def get_channel_name_from_id(channel_id):
    
    db = Database(get_db_path())

    return db.execute(f"SELECT channel_name FROM Channels WHERE channel_id = ?", [channel_id]).fetchone()[0]

def get_channel_name_from_video_id(video_id):
    
    db = Database(get_db_path())

    return db.execute(f"SELECT channel_name FROM Channels WHERE channel_id = (SELECT channel_id FROM Videos WHERE video_id = ?)", [video_id]).fetchone()[0]


# delete all videos, subtitles, and embeddings associated with channel
def delete_channel(channel_id):
    
    from .utils import check_ss_enabled
    from .vector_search import delete_channel_from_chroma 

    if check_ss_enabled(channel_id):
        delete_channel_from_chroma(channel_id)

    conn = sqlite3.connect(get_db_path())
    cur = conn.cursor()

    cur.execute("DELETE FROM Channels WHERE channel_id = ?", (channel_id,))

    # make sure to delete all subtitles and embeddings before videos  
    cur.execute("DELETE FROM Subtitles WHERE video_id IN (SELECT video_id FROM Videos WHERE channel_id = ?)", (channel_id,))

    cur.execute("DELETE FROM Videos WHERE channel_id = ?", (channel_id,))

    cur.execute("DELETE FROM SemanticSearchEnabled WHERE channel_id = ?", (channel_id,))

    conn.commit()
    conn.close()



def get_channel_id_from_rowid(rowid):
    
    db = Database(get_db_path())

    res = db.execute(f"SELECT channel_id FROM Channels WHERE ROWID = ?", [rowid]).fetchone()

    if res is None:
        return None
    else:
        return res[0]


def get_channel_id_from_name(channel_name):
    
    db = Database(get_db_path())

    res = db.execute(f"SELECT channel_id FROM Channels WHERE channel_name = ?", [channel_name]).fetchall()

    console = Console()
    if len(res) > 1:
        table = Table(header_style="bold magenta")
        table.add_column("id", style="dim", width=5)
        table.add_column("channel_name")
        table.add_column("channel_url")

        channels = db.execute(f"SELECT ROWID, channel_name, channel_url FROM Channels WHERE channel_name = ?", [channel_name]).fetchall()
        for channel in channels:
            table.add_row(str(channel[0]), channel[1], channel[2])

        console.print(table)    
        show_message("multiple_channels_found")
    if len(res) == 0:
        return None
    else:
        return res[0][0]


# for listing specific channel 
def get_channel_list_by_id(channel_id):
    
    db = Database(get_db_path())

    return db.execute(f"SELECT ROWID, channel_name, channel_url FROM Channels WHERE channel_id = ?", [channel_id]).fetchall()


def check_if_channel_exists(channel_id):
    """
    Check if channel exists in the database
    """

    db = Database(get_db_path())

    res = db.execute(f"SELECT channel_id FROM Channels WHERE channel_id = ?", [channel_id]).fetchall()
    if len(res) > 0:
        return True
    else:
        return False

def get_num_vids(channel_id):
    
    db = Database(get_db_path())

    return db.execute(f"SELECT COUNT(*) FROM Videos WHERE channel_id = ?", [channel_id]).fetchone()[0]

def get_vid_ids_by_channel_id(channel_id):
    
    db = Database(get_db_path())

    return db.execute(f"SELECT video_id FROM Videos WHERE channel_id = ?", [channel_id]).fetchall()


def get_all_subs_by_channel_id(channel_id):
    
    db = Database(get_db_path())

    parsed_subs = []
    subs = db.execute("""
        SELECT s.subtitle_id, s.video_id, s.start_time, s.stop_time, s.text, v.channel_id
        FROM Subtitles s
        JOIN Videos v ON s.video_id = v.video_id
        WHERE v.channel_id = ?
        """, [channel_id]).fetchall()
    
    for sub in subs:
        split_subs = sub[4].strip().split(" ")
        if len(split_subs) > 0: 
            parsed_subs.append(sub)

    return parsed_subs

# get all subs where semantic search is enabled
def get_all_subs_by_channel_id_ss(channel_id):
    
    db = Database(get_db_path())

    parsed_subs = []
    subs = db.execute("""
        SELECT s.subtitle_id, s.video_id, s.timestamp, s.text 
        FROM Subtitles s
        JOIN Videos v ON s.video_id = v.video_id
        WHERE v.channel_id = ?
        """, [channel_id]).fetchall()
    
    for sub in subs:
        if len(sub[3].strip()) > 0:
            parsed_subs.append(sub)
    return parsed_subs


def get_transcript_by_video_id(video_id):
    
    db = Database(get_db_path())

    return db.execute(f"SELECT text FROM Subtitles WHERE video_id = ?", [video_id]).fetchall()


def get_subs_by_video_id(video_id):

    db = Database(get_db_path())

    return db.execute(f"SELECT start_time, stop_time, text FROM Subtitles WHERE video_id = ?", 
                      [video_id]).fetchall()
    



import unittest
import os
from yt_fts.export import export_fts

class TestExportFts(unittest.TestCase):
    def test_export_fts(self):
        # Setup
        text = "oiwfjnoibne"
        scope = "all"
        channel_id = "1"
        video_id = None 

        result = export_fts(text, scope, channel_id, video_id)
        self.assertIsNone(result, "Expected None when no matches are found")

if __name__ == '__main__':
    unittest.main()